@@ -4,7 +4,7 @@
 #if defined(__EMSCRIPTEN__)
 # include <emscripten.h>
 # define ZSV_CLI_EXPORT EMSCRIPTEN_KEEPALIVE
-# define CLI_MAIN zsv
+# define ZSV_CLI_MAIN zsv
 #else
 # define ZSV_CLI_EXPORT
 #endif
