@@ -1,3 +1,10 @@
+/*
+ * This file has been modified from its original form, in order to use the ZSV csv parser
+ * The preamble / disclaimer to the original file is included below
+ * The modifications to this file are subject to the same license (MIT) as the ZSV parser
+ * as described at https://github.com/liquidaty/zsv/blob/main/LICENSE
+ */
+
 /*
 ** 2016-05-28
 **
@@ -11,7 +18,7 @@
 ******************************************************************************
 **
 ** This file contains the implementation of an SQLite virtual table for
-** reading CSV files, modified to use the ZSV csv parser
+** reading CSV files
 **
 ** Usage:
 **
@@ -43,6 +50,7 @@ SQLITE_EXTENSION_INIT1
 #include <zsv.h>
 #include <zsv/utils/string.h>
 #include <zsv/utils/arg.h>
+#include <zsv/utils/prop.h>
 
 #ifndef SQLITE_OMIT_VIRTUALTABLE
 
@@ -101,6 +109,7 @@ typedef struct zsvTable {
   sqlite3_vtab base;              /* Base class.  Must be first */
   char *zFilename;                /* Name of the CSV file */
   struct zsv_opts parser_opts;
+  char *opts_used;
   enum zsv_status parser_status;
   zsv_parser parser;
   struct zsv_vtab_cache header;
@@ -144,7 +153,7 @@ static int zsvtabCreate(
 
 static int add_row_to_cache(zsv_parser parser, struct zsv_vtab_cache *cache,
                             size_t row_id) {
-  size_t count = zsv_column_count(parser);
+  size_t count = zsv_cell_count(parser);
   struct zsv_vtab_cache_row *r = sqlite3_malloc(sizeof(*r));
   if(!r)
     return SQLITE_NOMEM;
@@ -164,17 +173,37 @@ static int add_row_to_cache(zsv_parser parser, struct zsv_vtab_cache *cache,
   cache->last = &r->next;
 
   r->column_count = count;
-  for(size_t i = 0; i < count; i++)
+  for(size_t i = 0; i < count; i++) {
     r->cells[i] = zsv_get_cell(parser, i);
+    if(r->id == 0) {
+      if(r->cells[i].len) {
+        void *copy = sqlite3_malloc(r->cells[i].len);
+        if(!copy) {
+          fprintf(stderr, "Out of memory!\n");
+          r->cells[i].len = 0;
+        } else {
+          memcpy(copy, r->cells[i].str, r->cells[i].len);
+          r->cells[i].str = copy; // zsv_memdup(r->cells[i].str, r->cells[i].len);
+        }
+      }
+    }
+  }
   return 0;
 }
 
+/* remove_row_from_cache: return 1 if row was removed */
 /* remove_row_from_cache: return 1 if row was removed */
 static int remove_row_from_cache(struct zsv_vtab_cache *cache) {
   if(cache->rows) {
-    struct zsv_vtab_cache_row *next = cache->rows->next;
-    sqlite3_free(cache->rows->cells);
-    sqlite3_free(cache->rows);
+    struct zsv_vtab_cache_row *r = cache->rows;
+    struct zsv_vtab_cache_row *next = r->next;
+    if(r->id == 0) {
+      for(size_t i = 0; i < r->column_count; i++)
+        if(r->cells[i].len)
+          sqlite3_free(r->cells[i].str);
+    }
+    sqlite3_free(r->cells);
+    sqlite3_free(r);
     if(!(cache->rows = next))
       cache->last = &cache->rows;
     return 1;
@@ -186,6 +215,7 @@ static void zsvTable_clear(struct zsvTable *z) {
   while(remove_row_from_cache(&z->data)) ;
   if(z->parser)
     zsv_delete(z->parser);
+  z->parser = NULL;
   z->rowCount = 0;
 }
 
@@ -194,6 +224,7 @@ static void zsvTable_delete(struct zsvTable *z) {
     zsvTable_clear(z);
     while(remove_row_from_cache(&z->header)) ;
     sqlite3_free(z->zFilename);
+    sqlite3_free(z->opts_used);
     sqlite3_free(z);
   }
 }
@@ -230,6 +261,7 @@ unsigned blank_column_name_count = 0;
 /**
  * Parameters:
  *    filename=FILENAME          Name of file containing CSV content
+ *    options_used=OPTIONS_USED  Used options (passed to zsv_new_with_properties())
  *    max_columns=N              Error out if we encounter more cols than this
  *
  * The number of columns in the first row of the input file determines the
@@ -244,13 +276,15 @@ static int zsvtabConnect(
 ){
   zsvTable *pNew = NULL;
   int rc = SQLITE_OK;        /* Result code from this routine */
-  static const char *azParam[] = {
-     "filename", "max_columns"
+  #define ZSVTABCONNECT_PARAM_MAX 3
+  static const char *azParam[ZSVTABCONNECT_PARAM_MAX] = {
+     "filename", "options_used", "max_columns"
   };
-  char *azPValue[1];         /* Parameter values */
+  char *azPValue[ZSVTABCONNECT_PARAM_MAX]; /* Parameter values */
 # define CSV_FILENAME (azPValue[0])
-  char *schema = NULL;
+# define ZSV_OPTS_USED (azPValue[1])
 
+  char *schema = NULL;
   pNew = zsvTable_new();
   if(!pNew)
     return SQLITE_NOMEM;
@@ -258,8 +292,6 @@ static int zsvtabConnect(
   pNew->parser_opts.max_columns = 2000; /* default max columns */
 
   (void)(_pAux);
-
-  assert( sizeof(azPValue)==sizeof(azParam) );
   memset(azPValue, 0, sizeof(azPValue));
 
   char *errmsg = NULL;
@@ -298,10 +330,14 @@ static int zsvtabConnect(
     goto zsvtab_connect_error;
   }
 
-  pNew->parser_opts.row = zsv_row_header;
+  pNew->parser_opts.row_handler = zsv_row_header;
   pNew->parser_opts.ctx = pNew;
-  if(!(pNew->parser = zsv_new(&pNew->parser_opts)))
-    goto zsvtab_connect_oom;
+  pNew->zFilename = CSV_FILENAME;
+  pNew->opts_used = ZSV_OPTS_USED;
+  CSV_FILENAME = ZSV_OPTS_USED = 0; // in use; don't free
+  if(zsv_new_with_properties(&pNew->parser_opts, pNew->zFilename, pNew->opts_used,
+                             &pNew->parser) != zsv_status_ok)
+    goto zsvtab_connect_error;
 
   pNew->parser_status = zsv_parse_more(pNew->parser);
   if(pNew->parser_status != zsv_status_ok &&
@@ -344,8 +380,7 @@ static int zsvtabConnect(
   if(!schema)
     goto zsvtab_connect_oom;
 
-  pNew->zFilename = CSV_FILENAME;
-  CSV_FILENAME = 0;
+
 #ifdef SQLITE_TEST
   pNew->tstFlags = tstFlags;
 #endif
@@ -434,7 +469,6 @@ static int zsvtabClose(sqlite3_vtab_cursor *cur){
   return SQLITE_OK;
 }
 
-
 /*
 ** Only a full table scan is supported.  So xFilter simply rewinds to
 ** the beginning.
@@ -453,9 +487,9 @@ static int zsvtabFilter(
   zsvTable_clear(pTab);
   fseek(pTab->parser_opts.stream, 0, SEEK_SET);
 
-  pTab->parser_opts.row = zsv_row_header;
+  pTab->parser_opts.row_handler = zsv_row_header;
   if(!(pTab->parser = zsv_new(&pTab->parser_opts)))
-    return SQLITE_NOMEM;
+    return SQLITE_ERROR;
   pTab->parser_status = zsv_parse_more(pTab->parser);
   return SQLITE_OK;
 }
@@ -539,14 +573,6 @@ sqlite3_module CsvModule = {
   0, 0, 0, 0 /* xSavepoint, xRelease, xRollbackTo, xShadowName */
 };
 
-/*
-sqlite3_module *get_csv_module() {
-  sqlite3_module *m = calloc(1, sizeof(*m));
-  *m = CsvModule;
-  return m;
-}
-*/
-
 #endif /* !defined(SQLITE_OMIT_VIRTUALTABLE) */
 
 
